#!/bin/sh

printf "\nIBM PBN for MQIPT\n"
printf "____________________\n"

base=`basename $0`
if [ "$base" = "$0" ]; then
  CD_BASE=`pwd`
else
  CD_BASE=`dirname $0`
  if [ "$CD_BASE" = "." ]; then
     CD_BASE=`pwd`
  fi
fi
cd $CD_BASE

INPUTLOCALE="n/a"
ISSILENT="no"

while [ -n "$1" ]; do
	case $1 in
		-silent)
		ISSILENT="yes";
		;;

		-locale)
		INPUTLOCALE=$2;
		shift;
		;;

		*)
		echo "Invalid command line parameter.";
		exit1;
		;;
	esac
	shift 1
done

if [ "$ISSILENT" = "no" ]; then
  printf "Proceed with installation? [yes] "; read DOINST
  if [ "$DOINST" = "n" -o "$DOINST" = "N" -o "$DOINST" = "no" -o "$DOINST" = "NO" -o "$DOINST" = "No" -o "$DOINST" = "nO" ]; then
	printf "  Installation aborted.\n"
	exit 1
  fi
fi

if [ "`whoami`" != "root" -a "$ISSILENT" = "no" ]; then
  printf "You are currently logged in as a user other than 'root'.\n"
  printf "Root-level access is required to install this product.\n"
  printf "Continue?  "; read cont
  if [ "$cont" != "y" -a "$cont" != "Y" -a "$cont" != "yes" -a "$cont" != "YES" ]; then
	printf "  Installation aborted.\n"
    exit 1
  fi

fi

if [ "$ISSILENT" = "no" ]; then
  printf "Have you read the license(license.txt) and agree to the terms ? [no] "; read DOINST
  if [ "$DOINST" = "y" -o "$DOINST" = "Y" -o "$DOINST" = "yes" -o "$DOINST" = "YES" -o "$DOINST" = "Yes" ]; then
	printf "  Continuing installation.\n"
  else
	printf "  Installation aborted.\n"
	exit 1
  fi
fi

COPYMADE="N"
UPGRADE="N"

oldfiles_dir="/opt/ibm/epbn/oldfiles"
if [ -f /etc/pagent.conf ]; then
	COPYMADE="Y"
	mkdir -p ${oldfiles_dir}
	cp /etc/pagent.conf ${oldfiles_dir}
fi
if [ "$COPYMADE" = "Y" ]; then
	printf "\n  Copies of your existing configuration files have been placed in:\n /opt/ibm/epbn/oldfiles.\n\n"
fi

rpm -qa | grep epbn > /dev/null 2>&1
if [ $? -eq 0 ]; then
	prev_version="Y"
	if [ "$ISSILENT" = "no" ]; then
		printf "There is a version of PBN for MQIPT already installed on \n this machine which will have to be removed.  Continue? [no] ";read CONT
	else
		CONT="yes"
	fi
	if [ "$CONT" = "yes" -o "$CONT" = "y" -o "$CONT" = "YES" -o "$CONT" = "Y" ] ; then

		pid=`ps -A | grep pagent | awk '{ print $1 }'`
		if [ "$pid" != "" ]; then
			kill -9 $pid
		fi
      		rpm -e `rpm -qa | grep epbn`
	else
    		printf "  Installation aborted.\n" 
		exit 1 
	fi

	rpm -qa | grep epbn > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		printf "\n  The existing product could not be uninstalled.\n"
		printf "  Installation aborted.\n"
		exit 1 
	fi
		
	printf "  Waiting for the process to finish shutting down...\n\n"
	while [ $[$(expr "$(ps -A | grep 'pagent')" : '.*pagent.*')] -ne 0 ]; do
    		sleep 1 s
	done

else	# no existing installation
	prev_version="N"
fi


printf "\nInstalling PBN for MQIPT\n"

printf ".Checking if the required packages are installed.\n"
#initialize the flags

dependency_check_failed=1
gskit_installed=0
ldap_clientd_installed=0
ld_so_conf_backup_file=""
ld_conf_done=0

while [ 1 -eq 1 ]
do

   #check if gskit is installed.
   printf "..Checking for IBM global security kit version 5 (gsk5bas)\n"
   rpm -q gsk5bas-5.0-4.58 > /dev/null 2>&1
   if [ $? -ne 0 ]; then
      rpm -i ./gsk5bas-5.0-4.58.i386.rpm
      if [ $? -ne 0 ]; then
         printf "...Install of gsk5bas-5.0-4.58 failed.\n"
         break
      else
         printf "...Install of gsk5bas-5.0-4.58 successfull\n"
         gskit_installed=1
      fi
   else
      printf "...gsk5bas-5.0-4.58 already installed.\n"
   fi

   #check if ldap client is installed.
   printf "..Checking for IBM ldap client (ldap-clientd)\n"
   rpm -qa | grep ldap-clientd-5.1-1 > /dev/null 2>&1
   if [ $? -ne 0 ]; then
	   rpm -i ./ldap-clientd-5.1-1.i386.rpm
	   if [ $? -ne 0 ]; then
         printf "...Install of ldap-clientd-5.1-1 failed\n"
         break
	   else
		   printf "...Install of ldap-clientd-5.1-1 successful\n"
         ldap_clientd_installed=1
	   fi
   else
	   printf "...ldap-clientd-5.1-1 already installed\n"
   fi

   #ldap client tweaking
   printf "..Checking for ldap-client in loadable module path\n"
   grep "/usr/ldap/lib" /etc/ld.so.conf > /dev/null 2>&1
   if [ $? -ne 0 ]; then
      echo "/usr/ldap/lib" >> /etc/ld.so.conf
      /sbin/ldconfig
	   if [ $? -ne 0 ]; then
         printf "...ldconfig of /etc/ld.so.conf failed."
         printf " Installation aborted\n"
         break
	   fi
   fi

   dependency_check_failed=0
   break
done

if [ $dependency_check_failed -eq 1 ]; then

   printf ".Dependency check failed. Performing cleanup\n"
   if [ $ld_conf_done -eq 1 ]; then
      printf ""
   fi

   if [ $ldap_clientd_installed -eq 1 ]; then
      printf "..Uninstalling ldap-clientd-5.1-1\n"
      rpm -e ldap-clientd-5.1-1
   fi

   if [ $gskit_installed -eq 1 ]; then
      printf "..Uninstalling gsk5bas-5.0-4.58\n"
      rpm -e gsk5bas-5.0-4.58
   fi

   printf "Aborting installation\n"
   exit 1
fi
 
printf "\n.Installing PBN for MQIPT.\n"
        rpm --ignorearch -i ./epbn-1.0-1.i686.rpm

if [ $? -eq 1 ]; then
        printf "\nUnable to install base code.\n"
        printf "Installation aborted.\n"
        exit 1
fi

/bin/cp /opt/ibm/epbn/samples/pagent.conf /etc/pagent.conf
if [ $? -ne 0 ]; then
  /bin/echo "Couldnot create the default config file for ibmPBN (/etc/pagent.conf). You may have to copy one from the /opt/ibm/epbn/samples/ directory."
fi
printf "..Install of PBN for MQIPT complete.\n"

install_dir="/opt/ibm/epbn"

PAGENT_STARTED="X"

printf "\n.Attempting to start the policy agent (pagent) for the first time...\n"
pid=`ps -A | grep pagent | awk '{ print $1 }'`
if [ "$pid" != "" ]; then
	kill -9 $pid
fi

/etc/rc.d/init.d/ibmPBN start > /dev/null 2>&1 

sleep 5
if [ -f /tmp/pagent.pid -a "`ps -A | grep pagent`" != "" ]; then
	printf "..The policy agent (pagent) has been started and is functional.\n\n"
	PAGENT_STARTED="Y"
else
	printf "\n..The policy agent (pagent) failed to start.  Please check the installation.\n"
        printf " Invoke '/etc/init.d/ibmPBN start' manually to see more information.\n"
	PAGENT_STARTED="N"
fi

if [ "${PAGENT_STARTED}" = "N" ] ; then
	exit 1
fi

if [ $[$(expr "$(ps -A | grep 'pagent')" : '.*pagent.*')] -ne 0 -o "$DOCSONLY" = "yes" ]; then
	printf "\nThe installation was successful.\n"
else
	printf "\nThe installation failed.\n"
	exit 1
fi

