/*********************************************************************/
/*                                                                   */
/* Part Name:          PAPIUSER.H                                    */
/*                                                                   */
/* Descriptive Name:   See Description Below                         */
/*                                                                   */
/* Component Name:     PAGN                                          */
/*                                                                   */
/* Copyright:    Licensed Materials - Property of IBM                */
/*               This product contains "Restricted Materials of IBM" */
/*               (C) Copyright IBM Corp. 2001, 2003.                 */
/*               All rights reserved.                                */
/*               US Government Users Restricted Rights -             */
/*               Use, duplication or diclosure restricted by         */
/*               GSA ADP Schedule Contract with IBM Corp.            */
/*               See IBM Copyright Instructions.                     */
/*                                                                   */
/* Status:       WAS 5.0/CSV1R6                                      */
/*                                                                   */
/* Function:     This file contains definitions for Policy API       */
/*               used by clients to access PAGENT.                   */
/*                                                                   */
/* Change Activity:                                                  */
/* CFD List:                                                         */
/*                                                                   */
/* Flag Reason   Release  Date   Origin    Description               */
/* ---- -------- -------- ------ --------  ------------------------  */
/* $Q0= D82113   WAS5.0   020513 HAWRYSZ : PerfMon enhancements      */
/* $QA= D83457   WAS5.0   020719 HAWRYSZ : change helper return codes*/
/* $QB= D83420   WAS5.0   020719 HAWRYSZ : change the activated time */
/*                                         field in SessionPerfInfo  */
/*                                         to a reserved field       */
/*                                                                   */
/* End CFD List:                                                     */
/*********************************************************************/
#ifndef _H_PAPIUSER
#define _H_PAPIUSER

#include <time.h>

#ifndef PAPI_HIDE_POLICY_OBJECTS
typedef unsigned short      Bit16;        /* Two octets              */
typedef unsigned long       Bit32;        /* Four octets             */
#endif

typedef unsigned long long  Bit64;        /* Eight Octets            */

#ifndef MAX_POLICY_NAME
#define MAX_POLICY_NAME           48
#endif
#ifndef MAX_IMAGE_NAME_SIZE
#define MAX_IMAGE_NAME_SIZE       9
#endif

/*********************************************************************/
/* Policy API Return Codes                                           */
/*********************************************************************/

#define PAPI_HELPER_RETURN_NULL       NULL /* Helper function returned
                                              on NULL input      @QAC*/
#define PAPI_OK                          0 /* Successful return code */
#define PAPI_HELPER_RETURN_ZERO          0 /* Helper function returned
                                              zero; this is an error,
                                              as opposed to the value
                                              zero that the function
                                              may return         @QAC*/
#define PAPI_NOK                         1 /* Generic Error          */
#define PAPI_CLIENT_ALREADY_INITIALIZED  4 /* PAPI client has already
                                              initialized with the
                                              policy agent           */
#define PAPI_CLIENT_ALREADY_REGISTERED   5 /* PAPI client has already
                                              registered with the
                                              policy agent           */
#define PAPI_FUNC_NOT_READY              8 /* A PAPI or Policy Agent
                                              function is not ready  */
#ifdef MULTIPLE_IMAGES
#define PAPI_CANNOT_SET_AFFINITY         9 /* Unable to set stack
                                              affinity               */
#endif /* -- MULTIPLE_IMAGES -- */

/*********************************************************************
   An environment error has occurred while execute PAPI
 *********************************************************************/

#define PAPI_UNAUTHORIZED_TO_REGISTER  15 /* Client NOT SUPERUSER    */
#define PAPI_MALLOC_FAILED             16 /* Could not allocate storage
                                             for PAPI or pasearch
                                             command                 */
#define PAPI_MALLOC_IN_PAGENT_FAILED   17 /* Pagent could not allocate
                                             storage                 */
#define PAPI_PAGENT_INTERNAL_ERROR     18 /* Internal error in Pagent*/
#define PAPI_INTERNAL_ERROR            19 /* Internal error in PAPI  */
#define PAPI_CLIENT_NOT_REGISTERED     20 /* client not successfully
                                             issue
                                             RegisterWithPolicyAPI   */
#define PAPI_NOT_VALID_AUTHORIZATION   21 /* Must be Either superuser
                                             or have security product
                                             authority               */
#define PAPI_VERSION_INCORRECT         22
#define PAPI_REFRESH_PERF_CACHE_FAILED 23 /* Pagent call to refresh the
                                             performance cache failed*/

/*********************************************************************/
/* Connection Errors                                                 */
/*********************************************************************/

#define PAPI_CONNECT_FAILED       30 /* Connect to server failed     */
#define PAPI_READ_FAILED          31 /* Read from buffer failed      */
#define PAPI_SOCKET_NOT_READABLE  32 /* socket is not readable       */
#define PAPI_WRITE_FAILED         33 /* Write to buffer failed       */
#define PAPI_READ_TIMEOUT         34 /* Read timed out               */
#define PAPI_SOCKET_FAILED        35 /* Could not open socket        */
#define PAPI_FCNTL_FAILED         36 /* fcntl call failed            */
#define PAPI_KERNEL_NOT_AVAILABLE 54 /* ioctl failed                 */

/*********************************************************************/
/* Policy API Return Errors For Invalid Parameters                   */
/*********************************************************************/

#define PAPI_INVALID_PARAMETER_VALUE  2   /* Parameter has invalid
                                             value                   */
#define PAPI_INVALID_ACCEPTABLE_CACHED_TIME  10 /* For Perf Collection,
                                                   must be >= min samp
                                                   interval          */
#define PAPI_PERF_COLL_TYPE_MISMATCH   11 /* Performance data not
                                             being collected was
                                             requested byt the API   */
#define PAPI_NULL_INPUT                50 /* A required input parameter
                                           to PAPI is not coded      */
#define PAPI_TCPIMAGE_NOT_VALID        51 /* The TcpImage was not
                                             a configured Pagent
                                             TcpImage                */
#define PAPI_TCPIMAGE_INVALID_LENGTH   52 /* TCPIMAGE > 8 char.      */
#define PAPI_FILTERNAME_INVALID_LENGTH 53 /* PolicyFilterName >
                                             47 character            */

/*********************************************************************/
/* END OF Policy API Return Codes                                    */
/*********************************************************************/

/*********************************************************************/
/* Constants for debug_options for papi_debug() API.                 */
/*********************************************************************/

#define PAPI_DEBUG_OFF            0  /* Debug Off                    */
#define PAPI_DEBUG_ON             1  /* Debug On                     */
#define PAPI_DEBUG_LAST           1  /* Last Valid Value             */

/*********************************************************************/
/* Log Levels                                                        */
/*********************************************************************/

#define PAPI_LOG_EMERG       0      /* A panic condition             */
#define PAPI_LOG_ALERT       1      /* Requires immediate action     */
#define PAPI_LOG_CRIT        2      /* Critical condition            */
#define PAPI_LOG_ERR         3      /* An error message              */
#define PAPI_LOG_WARNING     4      /* A warning message             */
#define PAPI_LOG_NOTICE      5      /* Requires special handling     */
#define PAPI_LOG_INFO        6      /* general info message          */
#define PAPI_LOG_DEBUG       7      /* program debug message         */

/*********************************************************************/
/* Constants for record type, used when retrieving performance data  */
/*********************************************************************/
#define PAPI_RULES            0x01
#define PAPI_ACTIONS          0x02
#define PAPI_SESSIONS         0x04
#define PAPI_PERF_ALL         0x07

/*********************************************************************/
/* papi_debug function defines                                       */
/*********************************************************************/

typedef struct {
   unsigned int    papiLogLevel;           /* Log Level              */
   void            *papiUserValue;         /* User Define Value      */
} papiLog_t;                               /* Input Log User Exit    */

typedef void (*papiLogUserExit_t)(papiLog_t *, ...);

typedef struct {
   unsigned int      papiDebugOpt;         /* Debug On/Off           */
   void              *papiUserValue;        /* User Define Value      */
   papiLogUserExit_t papiLogFunc;          /* Logging Function       */
} papiDebug_t;                             /* Input papi_debug()     */


/*********************************************************************/
/* Rule and Action Performance Information                           */
/*********************************************************************/
typedef struct {
   char            name[MAX_POLICY_NAME];  /* Rule / Action name     */
   Bit32           recordType;             /* Rule / Action          */
   Bit32           recordId;               /* Rule / Action Id       */
   time_t          firstActivated;         /* Time first activated   */
   Bit32           reserved1;              /* Reserved               */
   Bit64           bytesXmitted;           /* Total bytes transmitted*/
   Bit64           packetsXmitted;         /* Total packets
                                              transmitted            */
   Bit32           activeConnections;      /* Active connections
                                              count                  */
   Bit64           acceptedConnections;    /* Total accepted
                                              connections            */
   Bit32           smoothedRttAvg;         /* Average smoothed RTT   */
   Bit32           smoothedRttMdev;        /* MDEV of smoothed RTT   */
   Bit64           bytesRexmitted;         /* Total bytes
                                              retransmitted          */
   Bit64           packetsRexmitted;       /* Total packets
                                              retransmitted          */
   Bit32           smoothedConnDelayAvg;   /* Average smoothed conn
                                              delay                  */
   Bit32           smoothedConnDelayMdev;  /* MDEV of smoothed conn
                                              delay                  */
   Bit32           acceptQDelayAvg;        /* Average accept queue
                                              delay                  */
   Bit32           acceptQDelayMDev;       /* MDEV of accept queue
                                              delay                  */
   Bit64           packetsXmittedInProfile;/* Outbound in profile 
                                              packets count          */
   Bit64           bytesXmittedInProfile;  /* Outbound in profile bytes
                                              count                  */
} RulePerfInfo, ActionPerfInfo;

/*********************************************************************/
/* Session Performance Information                                   */
/*********************************************************************/
typedef struct {
   char            name[MAX_POLICY_NAME];  /* Session name           */
   Bit32           recordType;             /* Rule / Action          */
   Bit32           recordId;               /* Rule / Action Id       */
   Bit32           reserved0;              /* Reserved           @QBC*/
   Bit32           reserved1;              /* Reserved               */
   Bit64           bytesXmitted;           /* Total bytes transmitted*/
   Bit64           packetsXmitted;         /* Total packets
                                              transmitted            */
   Bit32           activeConnections;      /* Active connections
                                              count                  */
   Bit64           acceptedConnections;    /* Total accepted
                                              connections            */
   Bit32           smoothedRttAvg;         /* Average smoothed RTT   */
   Bit32           smoothedRttMdev;        /* MDEV of smoothed RTT   */
   Bit64           bytesRexmitted;         /* Total bytes
                                              retransmitted          */
   Bit64           packetsRexmitted;       /* Total packets
                                              retransmitted          */
   Bit32           smoothedConnDelayAvg;   /* Average smoothed conn
                                              delay                  */
   Bit32           smoothedConnDelayMdev;  /* MDEV of smoothed conn
                                              delay                  */
   Bit32           acceptQDelayAvg;        /* Average accept queue
                                              delay                  */
   Bit32           acceptQDelayMDev;       /* MDEV of accept queue
                                              delay                  */
   Bit64           packetsXmittedInProfile;/* Outbound in profile
                                              packets count          */
   Bit64           bytesXmittedInProfile;  /* Outbound in profile bytes
                                              count                  */
   Bit64           reserved2;              /* Reserved               */
   Bit64           reserved3;              /* Reserved               */
   Bit64           reserved4;              /* Reserved               */
   Bit64           reserved5;              /* Reserved               */
   Bit32           reserved6;              /* Reserved               */
   Bit32           maxMSS;                 /* Max MSS                */
   Bit32           classificationCount;    /* Classification count   */
   Bit32           clientConnections;      /* Number of client
                                              connections            */
   Bit32           smoothedMinTcpThroughput; /* Smoothed min tcp
                                                throughput           */
   Bit32           smoothedMaxTcpThroughput; /* Smoothed max tcp
                                                throughput           */
} SessionPerfInfo;

/*********************************************************************/
/* A session's name field contains a structure to uniquely identify  */
/* the session, since a session does not have an actual name         */
/* associated with it                                                */
/*********************************************************************/
typedef struct {
   Bit32 serverAddress;
   Bit16 serverPort;
   Bit32 clientAddress;
   Bit32 clientID;
} SessionName;

/*********************************************************************/
/* The PerfLogEntry structure will be used to log the performance    */
/* data to the log files.                                            */
/*********************************************************************/
#define PCOL_LOG_VERSION 1

typedef struct {
   time_t    recLogTime;
   int       recVersion;
}PerfLogHeader;

typedef struct {
   PerfLogHeader   header;
   SessionPerfInfo perfData;
} PerfLogEntry;

/*********************************************************************/
/* General API Function Prototypes                                   */
/*********************************************************************/
int papi_connect(void **, void *);
int papi_get_perf_data(void *, int, void *, int *, void *, void **);
int papi_disconnect(void *);
int papi_get_rules_count(void *);
int papi_get_actions_count(void *);
int papi_get_sessions_count(void *);
int papi_get_policy_instance(void *);
RulePerfInfo *papi_get_rule_perf_info(void *, int);
ActionPerfInfo *papi_get_action_perf_info(void *, int);
SessionPerfInfo *papi_get_session_perf_info(void *, int);
int papi_free_perf_data(void *);
int papi_debug (papiDebug_t *);


#endif  /* _H_PAPIUSER */

